Class: Nitro::FastCGI

Inherits:
Object
  • Object
show all
Defined in:
lib/nitro/adapter/fastcgi.rb

Overview

FastCGI Adaptor. FastCGI is a language independent, scalable, open extension to CGI that provides high performance without the limitations of server specific APIs.

Sessions

As FCGI is process based, you have cant use the default in-memory session store. For production web sites you should use the drb session store. Moreover, there is no need for DB connection pooling in Og.

Class Method Summary collapse

Class Method Details

.start(server) ⇒ Object



32
33
34
35
36
37
38
39
# File 'lib/nitro/adapter/fastcgi.rb', line 32

def self.start(server)
  FCGI.each do |cgi|
    begin
      Cgi.process(server, cgi, cgi.in, cgi.out)
      cgi.finish
    end
  end
end