Class: Nitro::OutputBuffer
- Inherits:
-
String
- Object
- String
- Nitro::OutputBuffer
- Defined in:
- lib/nitro/render.rb
Overview
– The output buffer. The output of a contoller action is accumulated in this buffer before sending this to the client as a HTTP Response.
TODO: Implement a FAST string (maybe in C) ++